#!/bin/bash
#
#    Generic AutoEx
#    Copyright (c) 2013 Sebastian Dufner
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

SCRIPTDIR="$(dirname "$(readlink -f ${BASH_SOURCE[0]})")"

GAx_CONFIG='./gaxconfig'
NULL=/dev/null

SHEET=
svn_options='--non-interactive --trust-server-cert'

################################################################################
#### Helpers ###################################################################
################################################################################

read_answer ()
{
    prompt="$1 "
    if [[ $2 = $LOCALE_Yes || $2 = $LOCALE_yes ]]
    then
        prompt="$prompt[$LOCALE_Yes/$LOCALE_no]"
    elif [[ $2 = $LOCALE_No || $2 = $LOCALE_no ]]
    then
        prompt="$prompt[$LOCALE_yes/$LOCALE_No]"
    else
        prompt="$prompt[$LOCALE_yes/$LOCALE_no]"
    fi

    read -p "$prompt : " answer
    if [[ "$answer"x = x ]]
    then
        answer=$2
    fi

    answer=`echo $answer | tr -s '[:upper:]' '[:lower:]'`
    if [[ $answer = $LOCALE_yes || $answer = $LOCALE_no ]]
    then
        return 0
    else
        return 1
    fi
}

read_score ()
{
    if [[ "$max_score"x = x ]]
    then
        read -p "$LOCALE_do_reward_score " score
    else
        eval "read -p \"$LOCALE_do_reward_score_max \" score"
    fi

    if echo $score | grep -E '^-?[[:digit:]][[:digit:]]*(\.[[:digit:]][[:digit:]]*)?' &>> $NULL
    then
        if [[ "$max_score"x = x ]]
        then
            if [[ `echo "$score < 0" | bc` = 1 ]]
            then
                return 0
            else
                return 1
            fi
        elif [[ `echo "${score#-} > $max_score" | bc` = 1 ]]
        then
            echo "$LOCALE_error $LOCALE_error_input_gt_max " >&2
            return 1
        else
            score=`echo "if ($score < 0) $max_score + $score else $score" | bc`
            return 0
        fi
    else
        echo "$LOCALE_error $LOCALE_error_only_numbers " >&2
        return 1
    fi
}

load_lang ()
{
    if [ -e "$SCRIPTDIR/locale-$lang" ]
    then
        source "$SCRIPTDIR/locale-$lang"
        localize_answers
    else
        return 1
    fi
}

localize_answers ()
{
    LOCALE_Yes=`echo $LOCALE_yes | tr -s '[:lower:]' '[:upper:]'`
    LOCALE_No=`echo $LOCALE_no | tr -s '[:lower:]' '[:upper:]'`
}

init_single_repo ()
{
        eval "echo \"$LOCALE_checkout\""
        svn co $url $svn_options &>> $NULL &
        pids="$pids $!"
}

init_repos ()
{
    pids=
    for id in $IDs
    do
        url=$URL/$id
        init_single_repo
    done

    url=$URL/validation
    init_single_repo
    url=$URL/tutor
    init_single_repo

    block

    echo
    echo "$LOCALE_checkouts_finished"
}

update_single_repo ()
{
    eval "echo \"$LOCALE_update\""
    svn up $id $svn_options &>> $NULL &
    pids="$pids $!"
}

update_repos ()
{
    pids=
    for id in $IDs
    do
        update_single_repo
    done

    id=validation
    update_single_repo
    id=tutor
    update_single_repo

    block

    echo "$LOCALE_updates_finished"
}

validate_against_standard_solution ()
{
    ccdir=scvssst
    if [ -e $ccdir ]
    then
        rm -rf $ccdir
    fi
    mkdir -p $ccdir/$SHEET/$ex

    cp -r $id/$SHEET/$ex/src $ccdir/$SHEET/$ex
    cp -r tutor/solutions/$SHEET/$ex/test $ccdir/$SHEET/$ex
    if [ -f tutor/solutions/$SHEET/$ex/build.xml ]
    then
        cp tutor/solutions/$SHEET/$ex/build.xml $ccdir/$SHEET/$ex
    else
        cp $id/$SHEET/$ex/build.xml $ccdir/$SHEET/$ex
    fi
    if [ -f tutor/solutions/$SHEET/$ex/checkstyle-config.xml ]
    then
        cp tutor/solutions/$SHEET/$ex/checkstyle-config.xml $ccdir/$SHEET/$ex
    else
        cp $id/$SHEET/$ex/checkstyle-config.xml $ccdir/$SHEET/$ex
    fi

    cd $ccdir/$SHEET/$ex

    eval "echo \"$LOCALE_validating_against_standard_solution\""
    eval "$VALIDATOR" &>> $NULL

    get_test_results true
    echo

    cd ../../..
}

get_test_results ()
{
    if [ -d report/junit ]
    then
        if [ "`sed '/<body[^>]*>/,/<\/body>/'\!'d' < report/junit/alltests-fails.html | hxselect ':first-child + .details td' 2> /dev/null`"x = x ]
        then
            if $1
            then
                green_text "$LOCALE_all_tests_passed"
            else
                echo "$LOCALE_all_tests_passed"
            fi
        else
            failed_tests="$(sed '/<body[^>]*>/,/<\/body>/'\!'d' < report/junit/alltests-fails.html | sed 's/<br>/<br\/>/g' | hxselect ':first-child + .details tr:nth-child(n+2)' 2> /dev/null | tr -d '\n' | sed 's/<\/code><\/td>/\n/g' | sed -r 's/^ *<td>[^<]*<\/td> *//g;s/<code>.*$//;s/<\/td>/ /g;s/<[^>]*>//g')"
            n=`echo "$failed_tests" | wc -l`
            failed_tests="`echo "$failed_tests" | awk '{ printf("%-19s %-24s %s\n", $1, $2, $3); $1=" "; $2=" "; $3=""; print $0 }'`"

            if $1
            then
                red_text "`eval "echo \"$LOCALE_tests_failed\""`"
                red_text "$failed_tests"
            else
                eval "echo \"$LOCALE_tests_failed\""
                echo "$failed_tests"
            fi
        fi

        return 0
    else
        if $1
        then
            red_text "$LOCALE_warning_no_tests"
        else
            echo "$LOCALE_warning_no_tests"
        fi

        return 1
    fi
    # sed '/<body[^>]*>/,/<\/body>/!d' < report/junit/overview-summary.html | hxselect ':first-child + .details' 2> /dev/null | tr -d '\n' | sed -r 's/<\/t(h|d)>/ /g;s/<[^>]*>//g' | awk '{ printf "    %-9s %-9s %-9s\n    %9s %9s %9s\n", $1, $2, $3, $7, $8, $9 }'
    # sed '/<body[^>]*>/,/<\/body>/!d' < report/junit/alltests-fails.html | hxselect ':first-child + .details' | tr -d '\n' | sed -r 's/<code>[^<]*<br\/>//g;s/<\/tr>/\n/g' | sed -r 's/(<[^>]*>[^<]*<\/[^>]*>[^<]*<[^>]*>[^<]*<\/[^>]*>[^<]*<[^>]*>[^<]*<\/[^>]*>[^<]*<[^>]*>[^<]*<\/[^>]*>[^<]*).*/\1/;s/<\/t(h|d)>/ /g;s/<[^>]*>//g;s/&lt;/</g;s/&gt;/>/g'
}

verify_commits ()
{
    echo "$LOCALE_verifying_commits"
    for id in $IDs
    do
        if svn status $id | grep "Feedback-$id-ex" &> $NULL
        then
            eval "echo \"$LOCALE_problem_in_working_copy\""
        fi

        echo "$LOCALE_done"
    done
}

write_config ()
{
    echo 'if [ "$IDs"x = x ]; then IDs='"'$IDs'"'; fi' > $GAx_CONFIG
    echo 'if [[ "$EDITOR"x = x ]]; then EDITOR='"'$EDITOR'"'; fi' >> $GAx_CONFIG
    echo 'if [[ "$VALIDATOR"x = x ]]; then VALIDATOR='"'$VALIDATOR'"'; fi' >> $GAx_CONFIG
    echo 'if [[ "$SVN_USER"x = x ]]; then SVN_USER='"'$SVN_USER'"'; fi' >> $GAx_CONFIG
    echo 'if [[ "$DAPHNE_URL"x = x ]]; then DAPHNE_URL='"'$DAPHNE_URL'"'; fi' >> $GAx_CONFIG
    echo 'if [[ "$DAPHNE_COURSE_PAGE"x = x ]]; then DAPHNE_COURSE_PAGE='"'$DAPHNE_COURSE_PAGE'"'; fi' >> $GAx_CONFIG
}

################################################################################
#### Utilities #################################################################
################################################################################

bell ()
{
    echo -en "\007"
}

block ()
{
    while ! [[ "$pids"x = x ]]
    do
        echo -n .

        for pid in $pids
        do
            if ! kill -0 $pid &>> $NULL
            then
                pids=`echo $pids | sed "s/$pid//g"`
            fi
        done

        sleep 1
    done
    bell
}

get_answer ()
{
    while ! read_answer "$1" $2
    do
        echo -n
    done
}

get_score ()
{
    while ! read_score
    do
        echo -n
    done
}

green_text ()
{
    echo -e "\e[32m$*\e[39m"
}

red_text ()
{
    echo -e "\e[31m$*\e[39m"
}

remove_escapes ()
{
    echo "$*" | sed 's/\x1B\[[0-9;]*[JKmsu]//g'
}

################################################################################
#### Program init functions ####################################################
################################################################################

get_variables ()
{
    if ! source $GAx_CONFIG &>> $NULL
    then
        eval "echo \"$LOCALE_warning_workspace_unitialized\"" >&2
        INIT=true
        echo
    fi

    if $INIT && [ "$IDs"x = x ]
    then
        echo "$LOCALE_error $LOCALE_error_missing_ids" >&2
        echo >&2
        print_usage
    fi

    if $INIT && [ "$URL"x = x ]
    then
        read -p "$LOCALE_do_specify_repository " URL
    fi

    if [[ "$DAPHNE_COURSE_PAGE"x = x ]]
    then
        read -p "$LOCALE_do_specify_course_page " DAPHNE_COURSE_PAGE
    fi
    DAPHNE_URL="`echo "$DAPHNE_COURSE_PAGE" | sed -r 's/^(http(s)?:\/\/[^\/]*).*$/\1/'`"

    if [[ "$EDITOR"x = x ]]
    then
        read -p "$LOCALE_do_specify_editor " EDITOR
    fi

    if [[ "$VALIDATOR"x = x ]]
    then
        read -p "$LOCALE_do_specify_validator " VALIDATOR
    fi

    if [[ "$SVN_USER"x = x ]]
    then
        read -p "$LOCALE_do_specify_username " SVN_USER
    fi
    if [[ "$SVN_PASSWORD"x = x ]]
    then
        read -sp "$LOCALE_do_specify_password " SVN_PASSWORD
    fi

    echo
    svn_options="$svn_options --username $SVN_USER --password $SVN_PASSWORD"
}

localize ()
{
    foo=false

    lang=default
    if load_lang
    then
        foo=true
    fi

    lang=`echo $LANG | cut -d '_' -f1`
    if load_lang
    then
        foo=true
    fi

    lang=`echo $lang | cut -d '.' -f1`
    if load_lang
    then
        foo=true
    fi

    lang=$LANG
    if load_lang
    then
        foo=true
    fi

    if $foo
    then
        return 0
    else
        echo "Unable to load language files, aborting!"
        exit 1
    fi
}

################################################################################
#### Core functions ############################################################
################################################################################

collect_feedback ()
{
    get_answer "$LOCALE_question_collect_feedback" $LOCALE_yes
    if [[ $answer = $LOCALE_yes ]]
    then
        feedbackfile="feedback/Feedback-`whoami`-$SHEET"
        mkdir -p "$feedbackfile"
        find $IDs -name "Feedback-*-$SHEET.txt" -exec cp '{}' "$feedbackfile/" \;
        tar -czf$feedbackfile.tar.gz -C`dirname $feedbackfile` `basename $feedbackfile`
        feedbackfile=$feedbackfile.tar.gz
        eval "echo \"$LOCALE_collected_feedback\""
    fi
}

commit_corrections ()
{
    get_answer "$LOCALE_question_commit_corrections" $LOCALE_yes
    if [[ $answer = $LOCALE_yes ]]
    then
        pids=
        for id in $IDs
        do
            eval "echo \"$LOCALE_commit\""
            svn ci $id -m "`svn status $id`" $svn_options &>> $NULL &
            pids="$pids $!"
        done

        block

        echo "$LOCALE_commits_finished"

        verify_commits
    fi
}

correct_submissions ()
{
    if ! $SKIP_CORRECTIONS
    then
        for id in $IDs
        do
            cd $id

            find -type f -name 'REPORT*.html' -exec svn rm '{}' --force \; &>> $NULL

            score_total=0

            feedbackfile="Feedback-$id-$SHEET.txt"
            echo -n > $feedbackfile
            svn add $feedbackfile &>> $NULL

            for ex in $EXs
            do
                echo

                if [ -d $SHEET/$ex ]
                then
                    cd $SHEET/$ex
                    eval "echo \"$LOCALE_validating\""
                    eval "$VALIDATOR" &>> $NULL
                    test_results="`get_test_results`"
                    cd ../..
                fi

                if [ -f ../tutor/solutions/$SHEET/instructions.org ]
                then
                    max_score=`grep -Ei "^\\* =$ex" ../tutor/solutions/$SHEET/instructions.org | sed -r 's/^.*= *\(([0-9][0-9]*(\.[0-9][0-9]*)?)p\).*$/\1/i'`
                else
                    max_score=
                fi

                if find $SHEET/$ex -type f -name 'REPORT*.html' 2>> $NULL | grep 'PASS' &>> $NULL
                then
                    if $CORRECTION_HINTS && [ -f ../tutor/solutions/$SHEET/instructions.org ]
                    then
                        echo
                        echo "$LOCALE_correction_hints"
                        "$SCRIPTDIR"/print_instructions.sh ../tutor/solutions/$SHEET/instructions.org "$ex" 2>> $NULL
                    fi

                    # sed '/Ex02_2/,/^\* /!d;s/^\* .*$//;s/^** /'`printf "\033[1m"`'/;s/$/'`printf "\033[0m"`'/;s/^- /  ‣ /;s/=\([a-zA-Z][a-zA-Z]*\)=/'`printf "\033[2m"`'\1'`printf "\033[0m"`'/g' < instructions.org
                    if [ -f $SHEET/$ex/report/coverage/index.html ]
                    then
                        coverage="`printf "%-39s%-18s%-10s\n" "Coverage" "Instructions" "Branches"; find $SHEET/$ex/report/coverage -type d -exec sed '/<body[^>]*>/,/<\/body>/'\!'d' '{}/index.html' \; 2> $NULL | hxselect 'tbody td:nth-child(-2n+5)' | sed -r 's/<[^>]*>/ /g;s/( *[^ ]* *[^ ]* *[^ ]*)/\1\n/g' | awk '{ printf("%12s%-20s |%19s |%13s\n", " ", $1, $2, $3)}'; hxselect 'tfoot td' < $SHEET/$ex/report/coverage/index.html | sed 's/<[^<]*>/ /g' | sed 's/ of /\//g' | awk '{ printf("%12s%-20s |%19s |%13s\n", " ", "Total", $3, $5)}'`"
                        echo "$coverage"
                    else
                        coverage=
                    fi

                    cd ..
                    validate_against_standard_solution
                    cd $id

                    feedback_temp='/tmp/gaxfeedback.tmp'
                    eval "echo -e \"\033[32m$LOCALE_exercise_valid\033[0m\""
                    echo "$test_results"

                    files="`find $SHEET/$ex/src $SHEET/$ex/test -type f -not -iwholename '*/.svn*' | tr '\n' ' '`"
                    files="$feedback_temp $files"
                    eval "$EDITOR $files" &>> $NULL &

                    while ! [ -e $feedback_temp ]
                    do
                        sleep 1
                    done

                    echo
                    get_score
                    score_total=`echo "$score_total + $score" | bc`

                    eval "echo \"$LOCALE_writing_feedback\""
                    echo $LOCALE_feedback_separator >> $feedbackfile
                    eval "printf ' %-59s%19s \n' \"$LOCALE_feedback_feedback\" '$score $LOCALE_feedback_points'" >> $feedbackfile
                    echo $LOCALE_feedback_separator >> $feedbackfile
                    echo "$test_results" | sed 's/\x1B\[[0-9;]*[JKmsu]//g' >> $feedbackfile
                    if ! [[ "$coverage"x = x ]]
                    then
                        echo "$coverage" >> $feedbackfile
                        echo >> $feedbackfile
                    fi
                    echo "$LOCALE_feedback_separator_dotted" >> $feedbackfile
                    cat $feedback_temp >> $feedbackfile
                    echo >> $feedbackfile

                    rm $feedback_temp
                else
                    if [ -d $SHEET/$ex ]
                    then
                        eval "echo -e \"\033[31m$LOCALE_exercise_invalid\033[0m\""
                    else
                        eval "echo -e \"\033[31m$LOCALE_exercise_missing\033[0m\""
                    fi
                    eval "echo \"$LOCALE_writing_feedback\""
                    echo $LOCALE_feedback_separator >> $feedbackfile
                    eval "printf ' %-59s%19s \n' \"$LOCALE_feedback_feedback\" '0 $LOCALE_feedback_points'" >> $feedbackfile
                    echo $LOCALE_feedback_separator >> $feedbackfile
                    if [ -d $SHEET/$ex ]
                    then
                        echo "$LOCALE_feedback_invalid" >> $feedbackfile
                        # hxselect 'body > pre *:nth-child(n+2)' < REPORT-ex02_1-2013-04-29-163226-INVALID.html | sed 's/<[^<]*>/\n/g'
                    else
                        echo "$LOCALE_feedback_missing" >> $feedbackfile
                    fi
                fi

                echo >> $feedbackfile
            done

            echo
            eval "echo -e \"\033[1m$LOCALE_total_score $score_total\033[0m\""
            echo
            echo $LOCALE_feedback_separator_total >> $feedbackfile
            eval "echo \"$LOCALE_feedback_total\"" >> $feedbackfile

            cd ..
        done
    fi
}

print_help_page ()
{
    echo "$LOCALE_help" >&2
    exit 1
}

print_usage ()
{
    echo "$LOCALE_usage" >&2
    exit 1
}

print_version ()
{
    echo "$LOCALE_version" >&2
    exit 1
}

init_workspace ()
{
    write_config
    init_repos
}

submit_scores ()
{
    get_answer "$LOCALE_question_submit_scores" $LOCALE_yes

    if [[ $answer = $LOCALE_yes ]]
    then
        "$SCRIPTDIR/post-scores.sh" "$SVN_PASSWORD"
    fi
}

update_workspace ()
{
    update_repos
}

################################################################################
#### Main code #################################################################
################################################################################

localize

# Parse options
CORRECTION_HINTS=true
INIT=false
SKIP_CORRECTIONS=false
while getopts ':e:hHi:l:p:s:Su:V:' opt
do
    case $opt in
        e)
            EDITOR="$OPTARG"
            ;;
        h)
            print_help_page
            ;;
        H)
            CORRECTION_HINTS=false
            ;;
        i)
            URL="$OPTARG"
            INIT=true
            ;;
        l)
            NULL="`readlink -f "$OPTARG"`"
            ;;
        p)
            SVN_PASSWORD="$OPTARG"
            ;;
        s)
            SHEET=$OPTARG
            ;;
        S)
            SKIP_CORRECTIONS=true
            ;;
        u)
            SVN_USER=$OPTARG
            ;;
        V)
            VALIDATOR="$OPTARG"
            ;;
        \?)
            if [[ $1 = '--help' ]]
            then
                print_help_page
            elif [[ $1 = '--version' ]]
            then
                print_version
            else
                eval "echo \"$LOCALE_option_invalid\"" >&2
                print_usage
            fi
            ;;
        :)
            eval "echo \"$LOCALE_option_requires_argument\"" >&2
            print_usage
            ;;
    esac
done
shift $(( $OPTIND - 1))

echo -n > $NULL

if ! [ "$*"x = x ]
then
    IDs="$*"
fi
get_variables

if $INIT
then
    init_workspace
else
    update_workspace
fi

if [[ "$SHEET"x = x ]]
then
    SHEET=`find validation -maxdepth 1 -type d -name 'ex*' -exec basename '{}' \; | sort | tail -n2 | head -n1`
fi
echo "LAST_SHEET='$SHEET'" >> $GAx_CONFIG

if ! $SKIP_CORRECTIONS && [[ "$SHEET" = "$LAST_SHEET" ]]
then
    get_answer "`eval "echo \"$LOCALE_question_correct_again\""`" $LOCALE_no
    if [[ $answer = $LOCALE_no ]]
    then
        SKIP_CORRECTIONS=true
    fi
fi

EXs=`find validation/$SHEET -maxdepth 1 -type d -name 'ex*_*' -exec basename '{}' \; | sort`

correct_submissions
commit_corrections
collect_feedback
submit_scores

echo
echo "$LOCALE_done"
