#!/bin/bash
#
# Author: Sebastian Dufner
# License: Public Domain
#
# Use this program at your own risk, it is distributed without any warranty.
#
#############
#
# This script will install 'Generic AutoEx' (GAX) to the
# default locations.
# If you only want to check for GAX's requirements, you can use
# './install check'

USER=`whoami`

COMMANDS='hxselect svn bash awk grep tar printf sed tr sort getopts find bc gzip cut curl'
req=true

SCRIPTDIR='~/.scripts'
SCRIPTDIREX='generic-autoex'
SHELLRC='~/.bashrc'
SCRIPTALIAS='gax'
MAINFILE='generic-autoex.sh'
ADDITIONAL_EXECS='post-scores.sh print_instructions.sh'
ADDITIONAL_FILES='locale-de locale-default'

if [ $USER == root ]
then
    SCRIPTDIR='/opt/der_scheme/'
    SHELLRC='/etc/bashrc'
fi

for command in $COMMANDS
do
    echo "Checking for $command..."
    type $command &> /dev/null
    eval "$command=$?; foo=\$$command"
    if [ $foo -ne 0 ]
    then
        req=false
    fi
done

echo

if $req
then
    echo "Your system meets all requirements."
else
    echo "Your system is missing the following commands:"
    for command in $COMMANDS
    do
        eval "foo=\$$command"
        if [ $foo -ne 0 ]
        then
            echo $command
        fi
    done

    exit 1
fi

if [[ $# = 1 ]]
then
    if [ $1 = check ]
    then
        exit
    fi
fi

if [[ ! -f $MAINFILE ]]
then
    echo "Couldn't find $MAINFILE, maybe you are in the wrong directory?"
    exit 1
fi

echo '-----------------------------------------------------------'
echo "Determining installation parameters..."
echo
echo "The main script will be copied to a subfolder of the directory where you keep your custom shell scripts."
read -ei "$SCRIPTDIR" -p "Scripting dir: " scriptdir
echo "An alias will be added to your shell profile. If you're not using bash or are not fine with adding this directly to your .bashrc, you'll have to change this."
read -ei "$SHELLRC" -p "Shell profile: " shellrc
read -ei "$SCRIPTALIAS" -p "Alias: " scriptalias

if [ "$scriptdir"x = x ]
then
    scriptdir=$SCRIPTDIR
fi

if [ "$shellrc"x = x ]
then
    shellrc=$SHELLRC
fi

if [ "$scriptalias"x = x ]
then
    scriptalias=$SCRIPTALIAS
fi

scriptdir="`eval echo ${scriptdir//>}`"
if [ $USER = root ]
then
    scriptdir="$scriptdir/$SCRIPTDIREX/bin"
else
    scriptdir="$scriptdir/$SCRIPTDIREX"
fi
shellrc="`eval echo ${shellrc//>}`"

echo '-----------------------------------------------------------'
echo "Installing..."
echo

mkdir -vp "$scriptdir"
cp -vf $MAINFILE "$scriptdir/$MAINFILE"

if [ $USER = root ]
then
    chmod -v 755 "$scriptdir/$MAINFILE"
else
    chmod -v +x "$scriptdir/$MAINFILE"
fi

for file in $ADDITIONAL_EXECS
do
    cp -vf $file "$scriptdir/$file"

    if [ $USER = root ]
    then
        chmod -v 755 "$scriptdir/$file"
    else
        chmod -v +x "$scriptdir/$file"
    fi
done

for file in $ADDITIONAL_FILES
do
    cp -vf $file "$scriptdir/$file"
done

echo "Adding alias $scriptalias to '$shellrc'."
if [ `grep "alias $scriptalias" $shellrc | wc -l` -gt 0 ]
then
    awk "/^alias $scriptalias/ { print \"alias $scriptalias='$scriptdir/$MAINFILE'\" } !/^alias $scriptalias/ { print }" $shellrc > /tmp/shellrc
    mv -f /tmp/shellrc $shellrc
else
    echo "alias $scriptalias='$scriptdir/$MAINFILE'" >> $shellrc
fi

echo '-----------------------------------------------------------'
echo "Installation complete!"
echo "Please restart your terminal or source $shellrc to make the alias work."
