#!/bin/bash
#
#    Copyright (c) 2013 Sebastian Dufner
#    This file is part of Generic AutoEx.
#
#    Generic AutoEx is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Generic AutoEx is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Generic AutoEx.  If not, see <http://www.gnu.org/licenses/>.

SCRIPTDIR="$(dirname "$(readlink -f ${BASH_SOURCE[0]})")"

COOKIES='./cookies'
#~ DAPHNE_URL='https://daphne-dev.informatik.uni-freiburg.de'
#~ DAPHNE_LOGIN_PAGE="$DAPHNE_URL/login/"
#~ DAPHNE_COURSE_PAGE="$DAPHNE_URL/ss2013/java/"
PASSWORD="$1"

GAX_CONFIG='./gaxconfig'
#~ LOG='./post.log'; echo -n > $LOG
LOG=/dev/null

curl_with_options ()
{
    curl -skLb "$COOKIES" -c "$COOKIES" --compressed "$@"
}

load_lang ()
{
    if [ -e "$SCRIPTDIR/locale-$lang" ]
    then
        source "$SCRIPTDIR/locale-$lang"
        localize_answers
    else
        return 1
    fi
}

localize_answers ()
{
    LOCALE_Yes=`echo $LOCALE_yes | tr -s '[:lower:]' '[:upper:]'`
    LOCALE_No=`echo $LOCALE_no | tr -s '[:lower:]' '[:upper:]'`
}

localize ()
{
    foo=false

    lang=default
    if load_lang
    then
        foo=true
    fi

    lang=`echo $LANG | cut -d '_' -f1`
    if load_lang
    then
        foo=true
    fi

    lang=`echo $lang | cut -d '.' -f1`
    if load_lang
    then
        foo=true
    fi

    lang=$LANG
    if load_lang
    then
        foo=true
    fi

    if $foo
    then
        return 0
    else
        echo "Unable to load language files, aborting!"
        exit 1
    fi
}

login ()
{
    echo "$LOCALE_logging_in"
    # Get Session Cookies and additional auth information
    auth="`curl_with_options $DAPHNE_LOGIN_PAGE | hxselect 'input[type=hidden][name]' | sed -r 's/<input( (type="hidden"|name="([^"]*)"|value="([^"]*)"|id="[^"]*"))*\/>/ --data-urlencode \3=\4/g;s/^ *//;s/ *$//'`"
    # Login
    curl_with_options -e $DAPHNE_LOGIN_PAGE $auth --data-urlencode "username=$SVN_USER" --data-urlencode "password=$PASSWORD" $DAPHNE_LOGIN_PAGE &>> $LOG
}

get_single_mapping ()
{
    file=`echo $1 | cut -d: -f1`
    file="$file/Feedback-$file-$LAST_SHEET.txt"

    if [ -f $file ]
    then
        echo "$1`tail -n1 $file | sed -r 's/[^0-9.]//g'`"
    else
        echo "${1}0"
    fi
}

get_user_mapping ()
{
    data="`curl_with_options $DAPHNE_COURSE_PAGE`"
    column=`echo "$data" | hxselect -s '\n' '#content > div > table th' | grep -Ein '\b'$LAST_SHEET'\b' | cut -d: -f1`
    column=$(( $column - 1 ))

    if [[ "$column"x = x ]]
    then
        echo 'Error: Unable to get column index. This should _never_ happen.' >&2
        exit 1
    fi

    mapping=`echo "$data" | hxselect -s '\n' '#content > div > table tbody tr' | tr -d '\r' | sed -r '/<tr class="(odd|even)">/,/<\/tr>/!d;s/^ *//;s/ *$//' | tr -d '\n' | sed -r 's/<\/tr>/\n/g;s/<tr[^>]*>//g' | sed -r 's/^<td><a[^>]*>//g;s/>[^<]*</></g;s/<(t|\/)[^>]*>//g;s/<a href="/ /g;s/">//g' | awk '{ printf("%s:%s:\n", $1,  $'$column') }'`
    for line in $mapping
    do
        get_single_mapping $line
    done
}

post_score ()
{
    auth="`curl_with_options ${DAPHNE_URL}$2 | hxselect 'input[type=hidden][name]' | sed -r 's/<input( (type="hidden"|name="([^"]*)"|value="([^"]*)"|id="[^"]*"))*\/>/ --data-urlencode \3=\4/g;s/^ *//;s/ *$//'`"
    curl_with_options $auth --data-urlencode points=$3 "${DAPHNE_URL}$2" &>> $LOG
}

post_mapping ()
{
    echo -n "$LOCALE_submitting_data"

    for entry in $1
    do
        post_score `echo $entry | tr ':' ' '`
        echo -n .
    done

    echo
}

if ! . $GAX_CONFIG
then
    echo no config
    exit 1
fi
DAPHNE_LOGIN_PAGE="$DAPHNE_URL/login/"

if [[ "$PASSWORD"x = x ]]
then
    read -sp "Password: " PASSWORD
fi

localize
login
post_mapping "`get_user_mapping`"
#~ get_user_mapping

# Tear down
rm "$COOKIES"
